package cds;

import java.util.List;
import java.util.ArrayList;

/**
 * CdsDataRace01 is intended to show possible data races when mutual exclusion
 * is not guaranteed. The provided solution is not correct: The problem relies
 * in the way the nested class CounterAccessingInt is implemented.
 */
public class CdsDataRace01 {

    // the number of competing threads to use is set via COMPETITORSNUM       
    static final int COMPETITORSNUM = 4; // number of competing threads

    private static final SharedCounter shCo = new CounterAccessingInt();

    /**
     *
     * @param args
     */
    public static void main(String[] args) {

        List<Competitor> competitors = new ArrayList<>();

        //all the competing threads are created
        for (int i = 0; i++ < COMPETITORSNUM; competitors.add(new Competitor(shCo)));
        //all the competing threads are started
        for (Competitor cmpt : competitors) {
            cmpt.start();
        }
        //using lambdas:
        //competitors.forEach((c)->c.start());
    }

    /**
     * Implementation of SharedCounter. This class is not thread-safe because
     * the read and write operations on 'count' are not performed atomically.
     */
    static class CounterAccessingInt implements SharedCounter {

        /**
        * This private field is the actual counter variable.
        */
        private int count = 0;
        
        private final java.util.Random myPRNG = new java.util.Random();

        @Override
        public int get() {

            int res = count;

            // the following pause is inserted to better uncover data races
            // in accessing 'count'. 
            try {
                Thread.sleep(myPRNG.nextInt(MAXSLEEP));
            } catch (InterruptedException e) {
            }

            count++;

            return res;
        }

        @Override
        public void reset(int initValue) {
            count = initValue;
        }
    }

}
