package cds;

import java.util.List;
import java.util.ArrayList;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * CdsDataRace01Fixed is intended to show a correct implementation/use of a
 * SharedCounter. The provided solution is correct, because of the semantics of
 * AtomicInteger, used to hold the counter values in the nested class
 * CounterAccessingAtomicInt.
 */
public class CdsDataRace01Fixed {
    // the number of competing threads to use is set via COMPETITORSNUM

    static final int COMPETITORSNUM = 3; // number of competing threads

    private static final SharedCounter shCo = new CounterAccessingAtomicInt();

    public static void main(String[] args) {

        List<Competitor> competitors = new ArrayList<>();

        //all the competing threads are created
        for (int i = 0; i++ < COMPETITORSNUM; competitors.add(new Competitor(shCo)));
        //all the competing threads are started
        for(Competitor cmpt : competitors) cmpt.start();
        //using lambdas:
        //competitors.forEach((c)->c.start());   
    }

    static class CounterAccessingAtomicInt implements SharedCounter {

        private final AtomicInteger count = new AtomicInteger(0);
        
        private final java.util.Random myPRNG = new java.util.Random();

        /**
         * This method is thread-safe because the read and write operations on
         * 'count' are performed atomically, as allowed by AtomicInteger
         */
        @Override
        public int get() {
            int res = count.getAndIncrement();
            //the following delay cannot affect the uncovering of data races!
            try {
                Thread.sleep(myPRNG.nextInt(MAXSLEEP));
            } catch (InterruptedException e) {
            }
            return res;
        }

        @Override
        public void reset(int initValue) {
            count.set(initValue);
        }
    }

}
