package cds;

import java.util.List;
import java.util.ArrayList;

/**
 * CdsDataRace02 shows how ThreadsLocal&lt;T&gt; holds actually separate
 * versions of 'count' values on a per-thread basis, inside the nested class
 * CounterAccessingThreadLocal.
 */
public class CdsDataRace02 {
    // the number of competing threads to use is set via COMPETITORSNUM

    static final int COMPETITORSNUM = 3; // number of competing threads

    private static final SharedCounter shCo = new CounterAccessingThreadLocal();

    public static void main(String[] args) {

        List<Competitor> competitors = new ArrayList<>();

        //all the competing threads are created
        for (int i = 0; i++ < COMPETITORSNUM; competitors.add(new Competitor(shCo)));
        //all the competing threads are started
        for(Competitor cmpt : competitors) cmpt.start();
        //using lambdas:
        //competitors.forEach((c)->c.start());   
    }

    static class CounterAccessingThreadLocal implements SharedCounter {

        //this is not shared at all!!!
        private final ThreadLocal<Integer> count;
        
        private final java.util.Random myPRNG = new java.util.Random();

        CounterAccessingThreadLocal() {
            this.count = new ThreadLocal<Integer>() {
                @Override
                protected Integer initialValue() {
                    return 0;
                }
            };
        }

        /*
         * This method return separate sequences for each thread
         */
        @Override
        public int get() {
            int res = count.get();
            // the following pause is inserted to possibly uncover data races
            // in accessing 'count'. 
            try {
                Thread.sleep(myPRNG.nextInt(MAXSLEEP));
            } catch (InterruptedException e) {
            }
            count.set(res + 1);
            return res;
        }

        @Override
        public void reset(int initValue) {
            count.set(initValue);
        }
    }

}
