package cds;

import java.util.List;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.locks.Lock;

/**
 * CdsDataRace03, through its components, shows the behavior of two threads that
 * concurrently access a shared data structure (a Map). Mutual exclusion is
 * guaranteed by the use of a lock implementing the Peterson's solution: pay
 * attention, it works only for two threads!
 */
public class CdsDataRace03 {
    // the number of competing threads to use is set via COMPETITORSNUM

    static final int COMPETITORSNUM = 2; // number of competing threads

    static Map<Integer, Integer> myMap = new HashMap<>();
    static Lock lock = new PetersonLock();
    //static Lock lock = new java.util.concurrent.locks.ReentrantLock();

    public static void main(String[] args) {

        List<MapCompetitor> competitors = new ArrayList<>();

        //all the competing threads are created
        for (int i = 0; i++ < COMPETITORSNUM; competitors.add(
                new MapCompetitor(myMap, lock)));
        //all the competing threads are started
        for(MapCompetitor cmpt : competitors) cmpt.start();
        //using lambdas:
        //competitors.forEach((c)->c.start());   

        //the main thread waits for all the spawn threads to complete
        try {
            //for (int i = 0; i < competitors.length; competitors[i++].join());
            for(MapCompetitor cmpt : competitors) cmpt.join();

            System.out.println("Finally, the content of the map is the following:");
            for (Map.Entry<Integer, Integer> e : myMap.entrySet()) {
                System.out.println(Integer.toString(e.getKey()) + ":" + Integer.toString(e.getValue()));
            }
        } catch (InterruptedException e) {
        }

    }
}
