package cds;

/**
 * Interface for any class that implements an integer shared counter.
 */
public interface SharedCounter {

    /**
     * The maximum time to sleep between reading and writing to 'count' in the
     * 'get' method
     */
    static final int MAXSLEEP = 1;

    /**
     * Gets the value from the shared counter. Getting a value has to determine
     * the increment of the counter as well.
     *
     * @return value read from the shared counter
     */
    public int get();

    /**
     * Resets the counter value. To set the counter to the value passed as
     * argument.
     *
     * @param initValue value to be written onto the counter at resetting
     */
    public void reset(int initValue);
}
