package cds;

/**
 * Class to assign a unique ID to threads in a program. It is intended to be
 * used invoking the static method <code>get()</code> to obtain a sequential
 * index for the invoking thread; any subsequent invocation from the same thread
 * will get the same result.
 */
import java.util.concurrent.atomic.AtomicInteger;

public class ThreadSeqID {

    /**
     * The next thread ID to be assigned
     */
    private static final AtomicInteger nextID = new AtomicInteger(0);

    /**
     * My thread-local ID.
     *
     */
    private static final ThreadLocal<Integer> threadID = new ThreadLocal<Integer>() {
        @Override
        protected Integer initialValue() {
            return nextID.getAndIncrement();
        }

    };

    public static int get() {
        return threadID.get();
    }

    public static void reset() {
        nextID.set(0);
    }

}
